function groupByTopPosition(elements) {
  var groups = {};
  $.each(elements, function(index, element) {
    var top = $(element).position().top;
    if (groups[top] === undefined) {
      groups[top] = [];
    }
    groups[top].push(element);
  });
  return groups;
}

function filterContentBody() {
  var pattern = searchPattern.get();
  var onlyNames = searchPattern.isOnlyNames();
  var allComponentsPresets = $('.component-presets');
  $.each(allComponentsPresets, function(_, componentPresets) {
    var presetLists = $(componentPresets).find('.preset-list');
    $.each(presetLists, function(_, presetList) {
      var presetItems = $(presetList).find('.preset-item');
      $.each(presetItems, function(_, presetItem) {
        if (isPresetMatches(presetItem, pattern, onlyNames)) {
          $(presetItem).show();
        } else {
          $(presetItem).hide();
        }
      });
      if (allHidden(presetItems)) {
        $(presetList).hide();
      } else {
        $(presetList).show();
      }
    });
    if (allHidden(presetLists)) {
      $(componentPresets).hide();
    } else {
      $(componentPresets).show();
    }
  });
  if (allHidden(allComponentsPresets)) {
    $('#nothing-found').show();
  } else {
    $('#nothing-found').hide();
  }
  equalizePresetItemHeights();
  searchPattern.save();
}

function isPresetMatches(presetItem, pattern, onlyNames) {
  var isMatches;
  var name = $(presetItem).attr('data-name').toLowerCase();
  isMatches = name.match(pattern);
  // Matching by type and id:
  if (isMatches === null && onlyNames !== true) {
    var cid   = $(presetItem).attr('data-cid').toLowerCase().replace(/[_0-9]+/g, ' ');
    var ctype = $(presetItem).attr('data-ctype').toLowerCase();
    var synonyms = (ctype === 'regular') ? 'standards' : '';
    if (model.data.currentLocale !== 'en-US') {
      synonyms += (ctype === 'regular') ? model.data.text.presets_standard.toLowerCase() : '';
      synonyms += (ctype === 'wallcutter') ? model.data.text.presets_cutters.toLowerCase() : '';
    }
    isMatches = (name + cid + ctype + 's' + synonyms).match(pattern);
  }
  return (isMatches !== null);
}

var refreshingBody = false;
function requestFilterContentBody() {
  if (refreshingBody) {
    return true;
  }
  refreshingBody = true;
  utils.wait(15).done(function() {
    refreshingBody = false;
    filterContentBody();
  });
  return true;
}

var searchPattern = {
  'get': function() {
    var that = this;
    var inputValue = this.getRawString().toLowerCase();
    var inputValueCut = inputValue.replace(/(^\ +|\ +$)/g, '');
    var words = inputValueCut.split(' ');
    var searchTerms = $.map(words, function(str) {
      if (that.isOnlyNames() !== true) {
        str = str.replace(/flex/g, '');
      }
      var str_escaped = str.replace(/[\-\[\]\/\{\}\(\)\*\+\?\.\\\^\$\|]/g, '\\$&');
      return '(?=.*' + str_escaped + ')';
    });
    var pattern = new RegExp(searchTerms.join('') + '.*');
    return pattern;
  },
  'getRawString': function() {
    return $('#form-control-search').val();
  },
  'set': function(pattern) {
    $('#form-control-search').val(pattern);
    $('#form-control-search').trigger('change');
  },
  'clear': function() {
    var $searchInput = $('#form-control-search');
    var isEmpty = !($searchInput.val());
    if (!isEmpty) {
      $searchInput.val('');
      $searchInput.trigger('cleared');
    }
  },
  'save': function() {
    var that = this;
    if (that.saveSearchRequestDfr) {
      that.saveSearchRequestDfr.reject();
    }
    that.saveSearchRequestDfr = utils.wait(50);
    that.saveSearchRequestDfr.done(function() {
      that.saveImmediately();
    });
  },
  'saveImmediately': function() {
    var strPat = this.getRawString();
    var onlyNames = this.isOnlyNames();
    ruby.send('save_search_query', [strPat, onlyNames]);
  },
  'isOnlyNames': function() {
    return $('#checkbox-search-by-name').prop('checked');
  },
  'setOnlyNames': function(newValue) {
    $('#checkbox-search-by-name').prop('checked', newValue);
  }
};

function allHidden(array) {
  var isHidden;
  var result = true;
  $.each(array, function(index, elem) {
    isHidden = $(elem).css('display') === 'none';
    result = result && isHidden;
  });
  return result;
}

function findMaxHeight(elements) {
  var maxHeight = 0;
  $.each(elements, function(index, element) {
    var elemHeight = $(element).height('auto').height();
    if (elemHeight > maxHeight) {
      maxHeight = elemHeight;
    }
  });
  return maxHeight;
}

function equalizePresetItemHeights() {
  var presetItems = $('.preset-item');
  var groups = groupByTopPosition(presetItems);
  $.each(groups, function(top, elements) {
    var maxHeight = findMaxHeight(elements);
    $.each(elements, function(index, element) {
      $(element).height(maxHeight);
    });
  });
}

function renamePreset($inputNewName) {
  var newName = $inputNewName.val();
  if (newName !== undefined && newName.replace(/\ /g,'') !== '') {
    var cid = $inputNewName.attr('data-cid');
    var ctype = $inputNewName.attr('data-ctype');
    var currentName = String($inputNewName.attr('data-name'));
    ruby.send('rename_preset', [cid, ctype, currentName, newName])
      .done(function() {
        view.refresh();
      });
  } else {
    utils.messagebox('Preset name can\'t be empty.')
      .done(function() {
        view.refresh();
      });
  }
}

function isSpecialKey(keyCode) {
  var isSpecial = keyCode === 13 || //ENTER
                  keyCode === 27 || // ESC
                  keyCode === 8;    // BACKSPACE
  return isSpecial;
}

var invalidNameChars = [
  '/', '?', '<', '>', '\\',
  ':', '*', '|', '"', '.'
];

function isValidNameCharacter(chr) {
  var isValid = invalidNameChars.indexOf(chr) < 0;
  return isValid;
}

function canReplaceWithSelected(preset) {
  var selected = model.data.selected;
  return (selected.canSaveAsPreset &&
          selected.cid === preset.cid &&
          selected.ctype === preset.ctype &&
          preset.vendor !== 'vendor');
}

function addClearButtonToSearchFields() {
  $('.clear-icon').css('background-image', 'url(' + model.data.icons.clear_icon + ')');
}

function buildMenu() {
  var menu = new BootstrapMenu('.preset-item', {
    fetchElementData: function($presetItem) {
      return {
        'domObj':     $presetItem,
        'isDefault':  $presetItem.attr('data-default'),
        'isOriginal': $presetItem.attr('data-original'),
        'cid':        $presetItem.attr('data-cid'),
        'ctype':      $presetItem.attr('data-ctype'),
        'name':       $presetItem.attr('data-name'),
        'vendor':     $presetItem.attr('data-vendor')
      };
    },
    actions: [
      {
        name: model.data.text.presets_menu_set_as_default,
        isEnabled: function(preset) {
          return !(preset.isDefault);
        },
        onClick: function(preset) {
          if (preset.isOriginal) {
            ruby.send('reset_default_preset', [preset.cid, preset.ctype])
              .done(equalizePresetItemHeights);
          } else {
            ruby.send('set_default_preset', [preset.cid, preset.ctype, preset.name, preset.vendor])
              .done(equalizePresetItemHeights);
          }
        }
      },
      {
        name: model.data.text.presets_menu_rename,
        isEnabled: function(preset) {
          return !(preset.isOriginal) && (preset.vendor !== 'vendor');
        },
        onClick: function(preset) {
          var presetName = preset.domObj.find('.preset-name');
          var presetNewNameInput = view.template.presetNewNameInput({
            'cid':   preset.cid,
            'ctype': preset.ctype,
            'name':  preset.name
          });
          presetName.replaceWith(presetNewNameInput);
          preset.domObj.find('.input-new-name').select();
        }
      },
      {
        name: model.data.text.presets_menu_replace,
        isEnabled: function(preset) {
          return !(preset.isOriginal) && canReplaceWithSelected(preset);
        },
        onClick: function(preset) {
          ruby.send('replace_with_selected', [preset.cid, preset.ctype, preset.name]);
        }
      },
      {
        name: model.data.text.presets_menu_delete,
        isEnabled: function(preset) {
          return !(preset.isOriginal);
        },
        onClick: function(preset) {
          ruby.send('delete_preset', [preset.cid, preset.ctype, preset.name, preset.vendor]);
        }
      }
    ]
  });

  // Adding data fetch before showing menu.
  // We need this to check if "Replace with selected"
  // menu item is enabled.
  menu.open = function($openTarget, ev) {
    var that = this;
    model.fetchData().done(function() {
      BootstrapMenu.prototype.open.call(that, $openTarget, ev);
    });
  };
}

function focusUndo() {
  var focusIsEmpty;
  var focused = $(':focus');
  if (ruby.isHtmlDialog) {
    focusIsEmpty = focused.length === 0;
  } else {
    focusIsEmpty = focused !== undefined &&
                   focused['0'] !== undefined &&
                   !($(focused['0']).is('input'));
  }
  if (focusIsEmpty) {
    searchPattern.clear();
  } else {
    focused.trigger('focusoutUndo');
  }
}

var view = new View({
  'filterKeys': true,
  'keyActions' : {
    27: focusUndo
  },
  //'contextMenuOff' : true
});

view.render = function() {
  $('#content-header').html(
    this.template.contentHeader(model.data)
  );
  $('#content-body').html(
    this.template.contentBody(model.data)
  );
  if (model.data.lastSearchQuery) {
    searchPattern.set(model.data.lastSearchQuery.pattern);
    searchPattern.setOnlyNames(model.data.lastSearchQuery.onlyNames);
  }
  this.hooks.render();
};

view.refreshContentBody = function() {
  var that = this;
  model.fetchData().done(function() {
    $('#content-body').html(
      that.template.contentBody(model.data)
    );
    that.hooks.render();
  });
};

view.refreshBtnSavePreset = function() {
  model.fetchData().done(function() {
    if (model.data.selected.canSaveAsPreset) {
      $('#btn-save-preset').prop('disabled', false);
    } else {
      $('#btn-save-preset').prop('disabled', true);
    }
  });
};

view.addHook('render', equalizePresetItemHeights);
view.addHook('render', buildMenu);
view.addHook('render', filterContentBody);
view.addHook('render', addClearButtonToSearchFields);

// Adds to the url current time.
// This will reload all thumbnails
// during view.refresh() call.
view.registerHelper('disableCaching', function(url) {
  return url + '?' + new Date().getTime();
});

view.on('click', '.btn-place-preset', function() {
  var skpPath = $(this).val();
  ruby.send('place_preset_instance', [skpPath]);
});

view.on('click', '#btn-save-preset', function() {
  ruby.send('save_preset_as');
});

view.on('focusout', '.input-new-name', function() {
  renamePreset($(this));
});

view.on('keypress', '.input-new-name', function(ev) {
  var ENTER_CODE = 13;
  var keyCode = ev.which;
  switch(keyCode) {
  case ENTER_CODE:
    $(this).blur();
    return;
  }
  var keyPressed = String.fromCharCode(keyCode);
  if (!isSpecialKey(keyCode) && !isValidNameCharacter(keyPressed)) {
    return false;
  }
});

view.on('focusoutUndo', '.input-new-name', function() {
  var currentName = $(this).attr('data-name');
  $(this).val(currentName);
  $(this).blur();
  return;
});

view.on('keypress', '#form-control-search', function(ev) {
  var keyCode = ev.which;
  var keyPressed = String.fromCharCode(keyCode);
  if (!isSpecialKey(keyCode) && !isValidNameCharacter(keyPressed)) {
    ev.preventDefault();
    return false;
  }
  return true;
});

view.on('focusoutUndo', '#form-control-search', function() {
  var isEmpty = !($(this).val());
  if (isEmpty) {
    $(this).blur();
  } else {
    searchPattern.clear();
  }
});

view.on('keydown', '#form-control-search', function(ev) {
  var BACKSPACE_CODE = 8;
  var keyCode = ev.which;
  var keyPressed = String.fromCharCode(keyCode);
  if (keyCode === BACKSPACE_CODE || isValidNameCharacter(keyPressed)) {
    requestFilterContentBody();
  }
});

view.on('cleared', '#form-control-search', function() {
  requestFilterContentBody();
});

view.on('change', '#checkbox-search-by-name', function() {
  requestFilterContentBody();
});

view.on('input change cleared', '#form-control-search', function() {
  var $this = $(this);
  var visible = Boolean($this.val());
  $('#search-clear').toggleClass('hidden', !visible);
});

view.on('click', '#search-clear', function() {
  searchPattern.clear();
  $('#form-control-search').focus();
});

$(window).resize(equalizePresetItemHeights);

view.refresh();
