// Async bridge to ruby.
// Creates $.Deferred for every
// Ruby call. Dialog callback should
// resolve it explicitly on return.
var ruby = {
  'isHtmlDialog': (window.sketchup !== undefined),
  'suMajorVersion': function() {
    return model.data.suMajorVersion;
  },
  'cid': -1,
  'callStatus': {},
  'resolve': function(cid, result) {
    ruby.callStatus[cid].resolve(result);
    $('#ft-dialog-args-' + cid).remove();
  },
  'call': function(fun, _args) {
    var currentCid;
    var domArgsId;
    var domArgs;
    var args = _args || [];
    if (ruby.isHtmlDialog) {
      currentCid = ++ruby.cid;
      ruby.callStatus[currentCid] = $.Deferred();
      args.push({
        onCompleted: function() {
          ruby.callStatus[currentCid].resolve();
        }
      });
      sketchup[fun].apply(this, args);
      return ruby.callStatus[currentCid];
    } else {
      // WebDialog backward compatibility code
      currentCid = ++ruby.cid;
      ruby.callStatus[currentCid] = $.Deferred();
      domArgsId = 'ft-dialog-args-' + currentCid;
      domArgs = '<input id="' + domArgsId + '" type="hidden"></input>';
      $('body').append(domArgs);
      $('#' + domArgsId).val(JSON.stringify(args));
      window.location = 'skp:' + fun + '@' + currentCid;
      return ruby.callStatus[currentCid];
    }
  },
  'send': function(msgID, args) {
    return ruby.call('send_ft_msg', [msgID, args]);
  }
};

// The object for transferring data from Ruby to JS.
// Correspondent Ruby view class should define a
// 'fetch_data' callback and store the data in
// the model.data field as JS object before
// resolving the call.
var model = {
  'data': {},
  'fetchData': function() {
    return ruby.call('fetch_data');
  },
  'setData': function(new_data) {
    model.data = new_data;
  }
};
